// This is the file to include in your code if you want access to the
// complete DList template class

// First, get the declaration for the base list class
#include "list.h"



// Linked list implementation
template <class Elem> class LList: public List<Elem> {
private:
  
  Link<Elem>* tail;       // Pointer to last Elem in list 
  Link<Elem>* fence;      // Last element on left side
  int leftcnt;            // Size of left partition
  int rightcnt;           // Size of right partition
  void init() {           // Intialization routine
    fence = tail = new Link<Elem>;
    leftcnt = rightcnt = 0;
  }

//////////////////////////////////////////////////////////////////////
  void removeall() {   // Return link nodes to free store
    
		while(tail->next !=tail)
	{
		fence=tail-> next;
		tail->next= tail->next->next;
		delete fence;
	}
	delete tail;
	tail=NULL;
}
/////////////////////////////////////////////////////////////////////// 

public:
  LList(int size=DefaultListSize) { init(); }
  ~LList() { removeall(); }  // Destructor
  void clear() { removeall(); init(); }
  bool insert(const Elem&);
  bool append(const Elem&);
  bool remove(Elem&);

  /////////////////////////////////////////////////////////
  void setStart()
    { fence = tail; rightcnt += leftcnt; leftcnt = 0; }
  //////////////////////////////////////////////////////////
  
  void setEnd()
    { fence = tail; leftcnt += rightcnt; rightcnt = 0; }
  void prev();

  ////////////////////////////////////////////////////
    void next() {
  
    fence = fence->next; rightcnt--; leftcnt++; }
  /////////////////////////////////////////////////

  int leftLength() const  { return leftcnt; }
  int rightLength() const { return rightcnt; }
  bool setPos(int pos);

  /////////////////////////////////////////////////////
  bool getValue(Elem& it) const {
    
    it = fence->next->element;
    return true;
  }
  //////////////////////////////////////////////////////

  void print() const;
};

// First are the functions not implemented in llist.h that
// are different from the singly linked list version.
template <class Elem> // Insert at front of right partition
bool LList<Elem>::insert(const Elem& item) {
  fence->next = new Link<Elem>(item, fence, fence->next);  

    fence->next->next->prev = fence->next;
  if (tail == fence)             // Appending new Elem
    tail = fence->next;          //   so set tail
  rightcnt++;                    // Added to right
  return true;
}

template <class Elem> // Append Elem to end of the list.
bool LList<Elem>::append(const Elem& item) {
  tail = tail->next = new Link<Elem>(item, tail, tail->next);
  rightcnt++;                    // Added to right
  return true;
}

// Remove and return first Elem in right partition
template <class Elem> bool LList<Elem>::remove(Elem& it) {
  if (fence->next == tail) return false; // (empty list)
  it = fence->next->element;       // Remember value
  Link<Elem>* ltemp = fence->next; // Remember link node
   ltemp->next->prev = fence;
  
  fence->next = ltemp->next;       // Remove from list
  delete ltemp;                    // Reclaim space
  rightcnt--;                      // Removed from right
  return true;
}

// Move fence one step left; no change if left is empty
template <class Elem> void LList<Elem>::prev() {

    fence = fence->prev; leftcnt--; rightcnt++; 
}

// Here are the functions that are identical to their singly linked 
// list counterparts.
// Set the size of left partition to pos
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


// Set the size of left partition to pos
template <class Elem> bool LList<Elem>::setPos(int pos) {
  if ((pos < 0) || (pos > rightcnt+leftcnt)) return false;
  fence = tail;
  for(int i=0; i<pos; i++) fence = fence->next;
  return true;
}



////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class Elem> void LList<Elem>::print() const {
  Link<Elem>* temp = tail->next;
  cout << "< ";
  while (temp != tail) {
    cout << temp->element << " ";
    temp = temp->next;
  }
  cout<<tail->element<< " ";
  cout << "| ";
  cout << ">\n";
}

////////////////////////////////////////////////////////////////////////