// This is the file to include in your code if you want access to the
// complete LList template class

// First, get the declaration for the base list class
#include "list.h"
#include "LINK.h"

// Linked list implementation
template <class Elem> class LList: public List<Elem> {
private:
  
  Link<Elem>* tail;        
  Link<Elem>* fence;      //Pointer to the current position
  int leftcnt;            // Size of left partition
  int rightcnt;           // Size of right partition
  //(int count;              // no need for left and right count, this counter count the #of element in the list)
  
  void init() {           // Intialization routine
    fence = tail = new Link<Elem>;
    tail->next=tail;            //we have 1 node pointing to itself (circular)
	leftcnt = rightcnt = 0;
	//count = 0;
  }


//////////////////////////////////////////////////////////////////////
  void removeall() {   // Return link nodes to free store
    
		while(tail->next !=tail)
	{
		fence=tail-> next;
		tail->next= tail->next->next;
		delete fence;
	}
	delete tail;
	tail=NULL;
}
/////////////////////////////////////////////////////////////////////// 


public:
  LList(int size=DefaultListSize) { init(); }
  ~LList() { removeall(); }  // Destructor
  void clear() { removeall(); init(); }
  bool insert(const Elem&);
  bool append(const Elem&); 
  bool remove(Elem&);

  /////////////////////////////////////////////////////////

  void setStart()
    { fence = tail; rightcnt += leftcnt; leftcnt = 0; }

  //////////////////////////////////////////////////////////
  
  void setEnd()
    { fence = tail; leftcnt += rightcnt; rightcnt = 0; }
  
  void prev();
  
  ////////////////////////////////////////////////////
  
  void next() {
  
    fence = fence->next; rightcnt--; leftcnt++; }
  
/////////////////////////////////////////////////

  int leftLength() const  { return leftcnt; }
  int rightLength() const { return rightcnt; }
  bool setPos(int pos);

  ///////////////////////////////////////////////////////////////////////

  bool getValue(Elem& it) const {
    
    it = fence->next->element;
    return true;
  }

  ////////////////////////////////////////////////////////////////////////////
  
  void print() const;
};



//////////////////////////////////////////////////////////////////////////////////

template <class Elem> // Insert at front of right partition
bool LList<Elem>::insert(const Elem& item) {
  fence->next = new Link<Elem>(item, fence->next);  
   //(no condition coz no end)
  rightcnt++;
  return true;
}
///////////////////////////////////////////////////////////////////////////////////


template <class Elem> // Append Elem : insert an element at the tail
bool LList<Elem>::append(const Elem& item) {
	tail=tail->next= new Link<Elem>(item, tail->next);
  rightcnt++;
  return true;
}

/////////////////////////////////////////////////////////////////////////////////


// Remove and return first Elem in right partition
template <class Elem> bool LList<Elem>::remove(Elem& it) {

 
  it = fence->next->element;       // Remember value
  Link<Elem>* ltemp = fence->next; // Remember link node
  fence->next = ltemp->next;       // Remove from list
  if (tail == ltemp) tail = fence; // Reset tail
  delete ltemp;                    // Reclaim space
  rightcnt--;
  return true;
}

///////////////////////////////////////////////////////////////////////////////
// Move fence one step left; no change if left is empty
template <class Elem> void LList<Elem>::prev() {
  Link<Elem>* temp = tail;
  while (temp->next!=fence) temp=temp->next;
  fence = temp;
  leftcnt--; rightcnt++;
}


//////////////////////////////////////////////////////////////////////////////


// Set the size of left partition to pos
template <class Elem> bool LList<Elem>::setPos(int pos) {
  if ((pos < 0) || (pos > rightcnt+leftcnt)) return false;
  fence = tail;
  for(int i=0; i<pos; i++) fence = fence->next;
  return true;
}



////////////////////////////////////////////////////////////////////////////

template <class Elem> void LList<Elem>::print() const {
  Link<Elem>* temp = tail->next;
  cout << "< ";
  while (temp != tail) {
    cout << temp->element << " ";
    temp = temp->next;
  }
  cout<<tail->element<< " ";
  cout << "| ";
  cout << ">\n";
}

/////////////////////////////////////////////////////////////////////////////